#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest
from os import environ, path, makedirs
from conan import ldap_authentication
from conan.test.configuration_file import ConfigurationFile
"""Validation for LDAP authenticator
"""

class TestAuthentication(unittest.TestCase):
    @classmethod
    def setUpClass(TestAuthentication):
        user_home = environ.get("HOME")
        name = path.join(user_home, '.conan_server', 'plugins', 'authentication')
        if not path.exists(name):
            makedirs(name=name)

    @classmethod
    def tearDownClass(TestAuthentication):
        del environ["CONAN_LDAP_AUTHENTICATION_CONFIG_FILE"]

    def setUp(self):
        ConfigurationFile.create_valid_config()

    def test_valid_login(self):
        authenticator = ldap_authentication.get_class()
        self.assertTrue(authenticator.valid_user(username="read-only-admin", password="password"))

    def test_invalid_username(self):
        authenticator = ldap_authentication.get_class()
        self.assertFalse(authenticator.valid_user(username="write-only-admin", password="password"))

    def test_invalid_password(self):
        authenticator = ldap_authentication.get_class()
        self.assertFalse(authenticator.valid_user(username="read-only-admin", password="foobar"))

    def test_invalid_server(self):
        ConfigurationFile.create_wrong_config()
        authenticator = ldap_authentication.get_class()
        self.assertFalse(authenticator.valid_user(username="read-only-admin", password="foobar"))

    def test_invalid_config(self):
        ConfigurationFile.create_invalid_config()
        authenticator = ldap_authentication.get_class()
        self.assertFalse(authenticator.valid_user(username="read-only-admin", password="foobar"))

    def test_empty_password(self):
        authenticator = ldap_authentication.get_class()
        self.assertFalse(authenticator.valid_user(username="read-only-admin", password=None))

    def test_empty_username(self):
        authenticator = ldap_authentication.get_class()
        self.assertFalse(authenticator.valid_user(username=None, password="password"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Virtual LDAP configuration
"""
from tempfile import NamedTemporaryFile
from os import environ, path


class ConfigurationFile(object):
    test_valid_config = '''
    [ldap]
    # LDAP server address
    host: ldap://ldap.forumsys.com
    # Distinguished name (DN) of the entry
    distinguished_name: cn=$username,dc=example,dc=com
    '''

    test_wrong_config = '''
    [ldap]
    # LDAP server address
    host: ldap://ldap.forumsys
    # Distinguished name (DN) of the entry
    distinguished_name: cn=$username,dc=example,dc=com
    '''

    test_invalid_config = '''
    [server]
    # LDAP server address
    host: ldap://ldap.forumsys.com
    # Distinguished name (DN) of the entry
    distinguished_name: cn=$username,dc=example,dc=com
    '''

    @staticmethod
    def create_config_file(file_content):
        temp_file = NamedTemporaryFile(prefix="ldap-authentication-", delete=False)
        with open(temp_file.name, 'w') as file:
            file.write(file_content)
        environ["CONAN_LDAP_AUTHENTICATION_CONFIG_FILE"] = temp_file.name

    @staticmethod
    def create_valid_config():
        ConfigurationFile.create_config_file(ConfigurationFile.test_valid_config)

    @staticmethod
    def create_wrong_config():
        ConfigurationFile.create_config_file(ConfigurationFile.test_wrong_config)

    @staticmethod
    def create_invalid_config():
        ConfigurationFile.create_config_file(ConfigurationFile.test_wrong_config)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest
import subprocess
import os
import signal
import time
from conans.client.conan_api import ConanAPIV1
import conans.errors
from conan.test.configuration_file import ConfigurationFile
"""Validate LDAP authentication by Conan Server
"""

class TestConanAuthentication(unittest.TestCase):
    conan_server_name = 'conan_server'
    pid = 0
    conan_api = None
    conan_server_conf_data = """
[server]
jwt_secret: MnpuzsExftskYGOMgaTYDKfw
jwt_expire_minutes: 120

ssl_enabled: False
port: 9300
public_port:
host_name: localhost

store_adapter: disk
authorize_timeout: 1800

disk_storage_path: ~/.conan_server/data
disk_authorize_timeout: 1800

updown_secret: NyiSWNWnwumTVpGpoANuyyhR

custom_authenticator: ldap_authentication

[write_permissions]

[read_permissions]
*/*@*/*: *

# Authentication type: [raw, ldap]
[auth]
type: users

[users]
"""

    @classmethod
    def setUpClass(TestConanAuthentication):
        conan_server_conf_path = os.path.join(os.path.expanduser('~'), '.conan_server', 'server.conf')
        if not os.path.exists(conan_server_conf_path):
            with open(conan_server_conf_path, 'w') as file:
                file.write(TestConanAuthentication.conan_server_conf_data)
        TestConanAuthentication.pid = subprocess.Popen(TestConanAuthentication.conan_server_name, stdout=subprocess.PIPE).pid
        time.sleep(3)
        TestConanAuthentication.conan_api, _, _ = ConanAPIV1.factory()
        TestConanAuthentication.conan_api.remote_add(remote="local", url="http://0.0.0.0:9300/")

    @classmethod
    def tearDownClass(TestConanAuthentication):
        os.kill(TestConanAuthentication.pid, signal.SIGTERM)
        TestConanAuthentication.conan_api.remote_remove(remote="local")

    def test_valid_ldap_login(self):
        TestConanAuthentication.conan_api.user(name="read-only-admin", password="password", remote="local")

    def test_invalid_ldap_login(self):
        try:
            TestConanAuthentication.conan_api.user(name="read-only-admin", password="foobar", remote="local")
            self.fail()
        except conans.errors.AuthenticationException as exception:
            self.assertEqual(str(exception), 'Wrong user or password. [Remote: local]')



from setuptools import setup
from setuptools import find_packages
from setuptools.command.install import install
from codecs import open
import os
"""LDAP authenticator for Conan project.

"""
__author__ = "Uilian Ries"
__license__ = "MIT"


class Path(object):

    @staticmethod
    def home():
        username = os.getenv("SUDO_USER", None)
        home_path = '~%s' % username if username else '~'
        return os.path.expanduser(home_path)


class Requirements(object):
    __requirements_path = "conan/requirements.txt"

    @staticmethod
    def get():
        """Retrieve all dependencies for this project

        :param filename:
        :return:
        """
        requirements = []
        with open(Requirements.__requirements_path) as req_file:
            for line in req_file.read().splitlines():
                if not line.strip().startswith("#"):
                    requirements.append(line)
        return requirements


class PostInstallCommand(install):
    """Override Install command to change plugin owner

    """
    def run(self):
        """If necessary, create plugin directory, install and change file owner

        :return: None
        """
        paths = ['.conan_server', 'plugins', 'authenticator']
        conan_path = Path.home()
        for path in paths:
            conan_path = os.path.join(conan_path, path)
            if not os.path.isdir(conan_path):
                self.__mkdir_sudo_user(conan_path)
        install.run(self)
        os.chown(os.path.join(conan_path, 'ldap_authentication.py'), self.__get_sudo_uid(), self.__get_sudo_gid())

    def __mkdir_sudo_user(self, path):
        """Create a directory and change owner if is running as sudo

        :param path: directory path to be created
        :return: None
        """
        os.mkdir(path)
        os.chown(path, self.__get_sudo_uid(), self.__get_sudo_gid())

    def __get_sudo_uid(self):
        """Get sudo UID from environment variables

        :return: sudo uid
        """
        return int(os.environ.get('SUDO_UID')) if 'SUDO_UID' in os.environ else os.getuid()

    def __get_sudo_gid(self):
        """Get sudo GID from environment variables

        :return: sudo gid
        """
        return int(os.environ.get('SUDO_GID')) if 'SUDO_GID' in os.environ else os.getgid()


setup(
    name='conan_ldap_authentication',
    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version="0.2.3",

    description='LDAP authenticator for Conan C/C++ package manager',

    # The project's main homepage.
    url='https://github.com/uilianries/conan-ldap-authentication',

    # Author details
    author='Uilian Ries',
    author_email='uilianries@gmail.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
    ],

    # What does your project relate to?
    keywords=['conan', 'ldap', 'package', 'authentication'],

    # You can just specify the packages manually here if your project is
    # simple. Or you can use find_packages().
    packages=find_packages(),

    # Alternatively, if you want to distribute just a my_module.py, uncomment
    # this:
    #   py_modules=["my_module"],

    # List run-time dependencies here.  These will be installed by pip when
    # your project is installed. For an analysis of "install_requires" vs pip's
    # requirements files see:
    # https://packaging.python.org/en/latest/requirements.html
    install_requires=Requirements.get(),

    # List additional groups of dependencies here (e.g. development
    # dependencies). You can install these using the following syntax,
    # for example:
    # $ pip install -e .[dev,test]
    extras_require={
    },

    # If there are data files included in your packages that need to be
    # installed, specify them here.  If using Python 2.6 or less, then these
    # have to be included in MANIFEST.in as well.
    package_data={
        'conan': ['*.txt'],
    },

    # Although 'package_data' is the preferred approach, in some case you may
    # need to place data files outside of your packages. See:
    # http://docs.python.org/3.4/distutils/setupscript.html#installing-additional-files # noqa
    # In this case, 'data_file' will be installed into '<sys.prefix>/my_data'
    data_files=[(os.path.join(Path.home(), '.conan_server', 'plugins', 'authenticator'), [os.path.join('conan', 'ldap_authentication.py')])],

    # Give access to write new files at authenticator directory
    cmdclass={'install': PostInstallCommand},
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import ldap
from configparser import ConfigParser
from os.path import isfile
from os.path import expanduser
from os.path import join
from os import getenv
from logging import warning
"""LDAP Authenticator for Conan Project.

This module offers LDAP authentication with Conan Server.

Example:
    The user can normally apply his username and password, as before

        $ conan user -p my_password my_username

Conan will call this authenticator to provide a custom validation by LDAP.

Settings:
    host: LDAP server address.
        e.g ldap://host:port

    distinguished_name: A DN is a sequence of relative distinguished names
                       (RDN) connected by commas.
        e.g. uid=my_username,ou=people,dc=example,dc=com

Dependencies:
    This project uses **PyLDAP** to connect LDAP server


   https://github.com/uilianries/conan-ldap-authenticator
"""
__author__ = "Uilian Ries"
__license__ = "MIT"


def get_class():
    """Entry point for Conan Server call this custom authenticator

    :return: The LDAP authenticator instance
    """
    LDAPConfigFile()
    return LDAPAuthenticator()


class LDAPConfigurationException(Exception):
    """Configuration Exception

    Prefix message for configuration
    """
    def __init__(self, value):
        self.value = "ERROR: Could not load configuration: %s " % value

    def __str__(self):
        return repr(self.value)


class LDAPConfigFile(object):
    """Create and retrieve configuration file

    """
    def __init__(self):
        LDAPConfigFile.__create(self.path())

    @staticmethod
    def path():
        """Retrieve configuration file path

        :return: Default file path
        """
        home = expanduser('~')
        default_path = join(home, ".conan_server", "ldap_authentication.conf")
        return getenv("CONAN_LDAP_AUTHENTICATION_CONFIG_FILE", default_path)

    @staticmethod
    def __create(filename):
        """Create default config file if **NOT** present

        :param filename: File path
        """
        if not isfile(filename):
            default_config_file = '''
[ldap]
# LDAP server address
host: ldap://ldap.forumsys.com
# Distinguished name (DN) of the entry
distinguished_name: cn=$username,dc=example,dc=com
'''
            with open(filename, "w") as text_file:
                text_file.write(default_config_file)


class LDAPConfiguration(object):
    """Configuration entity for LDAP authenticator

    Load config file and each property
    """
    def __init__(self, filename):
        LDAPConfiguration.__check_filename(filename)
        self.__config = ConfigParser()
        self.__config.read(filename)

    @staticmethod
    def __check_filename(filename):
        if not isfile(filename):
            raise LDAPConfigurationException("File name doest not exist")

    @property
    def host(self):
        return self.__config["ldap"]["host"]

    @property
    def dn(self):
        return self.__config["ldap"]["distinguished_name"]

    def dn_with_cn(self, username):
        return self.dn.replace("$username", username)


class LDAPAuthenticator(object):
    """Provide LDAP authentication by PyLDAP for Conan server.

    Settings should be present in default_ldap_authentication.py, this file is in same
    """
    def valid_user(self, username, password):
        result = False
        try:
            if not username:
                raise Exception('Empty username provided.')
            if not password:
                raise Exception('Empty password provided.')
            config_file = LDAPConfigFile().path()
            conf = LDAPConfiguration(config_file)
            connection = ldap.initialize(conf.host)
            connection.simple_bind_s(conf.dn_with_cn(username), password)
            result = True
        except Exception as error:
            warning(str(error))
        return result
